#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file's purpose is to test the various
# MultimediaParadise-related components.
# =========================================================================== # 
# This file is called:
#   $RSRC/multimedia_paradise/test/testing_multimedia_paradise_project.rb
# =========================================================================== #
require 'fileutils'
require 'colours/colours_e_autoinclude.rb'
require 'cliner'
require 'mountpoints'
require 'multimedia_paradise/autoinclude' # Include the main namespace first.
include MultimediaConversions
results = Mountpoints[]
if results.empty?
  results = '/Depot/'
end
if results.is_a? Array
  results = results.first unless results.empty?
end
# =========================================================================== #
# Some important constants come next.
# =========================================================================== #
BASE_DIR  = '/Depot/Temp/MultimediaParadise/'
MP3_FILE  = '/Depot/Audio/HansZimmer_BackdraftTheme.mp3'
TEST_FILE = MP3_FILE
MP4_FILE  = BASE_DIR+'test.mp4'
MAIN_FILE = MP4_FILE
# =========================================================================== #
# === mp4?
# =========================================================================== #
def mp4?; MP4_FILE; end
WAV_FILE  = BASE_DIR+'test.wav'
# =========================================================================== #
# === wav?
# =========================================================================== #
def wav?; WAV_FILE; end
_ = TEST_FILE
# =========================================================================== #
# Intro message.
# =========================================================================== #
cliner {
  e 'This project will try to test the Multimedia project.'
  e N+'For this to work, it will generate a base directory'
  e 'for all these tests now, at '+sdir(BASE_DIR)
}
e
FileUtils.mkdir_p(BASE_DIR) unless Dir.exist? BASE_DIR # Create the Base Dir.
FileUtils.cd(BASE_DIR)
# =========================================================================== #
# Assign the BASE_DIR to a directory target.
# =========================================================================== #
MultimediaParadise.set_output_directory(BASE_DIR)
cliner
unless File.exist? BASE_DIR+File.basename(TEST_FILE)
  e 'Now copying '+sfancy(TEST_FILE)+' to '+sdir(BASE_DIR+File.basename(TEST_FILE))
  FileUtils.copy(TEST_FILE, BASE_DIR+File.basename(TEST_FILE))
  cliner
end
# =========================================================================== #
# If the .mp4 file does not exist, we must copy it.
# =========================================================================== #
unless File.exist? MP4_FILE
  _ = (results+'/Video/Realvids/027_StarTrekII.mp4').squeeze '/'
  full_new_location = BASE_DIR+'test.mp4'
  e 'Now copying '+sfancy(_)+' to '+sdir(full_new_location)
  FileUtils.copy(_, full_new_location)
  cliner
end
# =========================================================================== #
# Identify a .mp4 file next:
# =========================================================================== #
e 'Now spitting out some information from '+sfancy(mp4?)+' using identify_video().'
identify_video MP4_FILE
e 'Ok, done.'
cliner
# =========================================================================== #
# === Trying to extract a video from a .mp4 file:
# =========================================================================== #
e 'Now trying to extract the video from '+sfancy(mp4?)+'.'
# stored_where = extract_video(MP4_FILE)
# e 'We should have stored in '+sfile(stored_where)+'.'
ewarn '^^^ The above has been disabled for now, as this takes too much time.'
cliner
# =========================================================================== #
# === Extract audio from the .mp4 file next.
# =========================================================================== #
e 'Now extracting audio from a '+sfancy('.mp4')+' file:'
extract_audio MP4_FILE, :pwd
cliner
e 'Now trying to convert to vob file from '+sfancy(MP4_FILE)
e 'Note: We currently skip this step because it takes quite a long time.'
# to_vob MP4_FILE
cliner
e 'We have finished testing the extracting-related features.'
cliner
e 'Now we will test merging some files:'
e 'Where we can find our VideoCollection:'
efancy '  '+video_collection?
cliner
e 'Now testing whether foobar.avi is considered a video file or not:'
e is_video? 'foobar.avi' # testmedia
cliner
e 'The next part is to test various conversions.'
# =========================================================================== #
# === test_various_conversions                              (conversions tag)
#
# The various mp3_to_* or *_to_* are audio-format conversions.
#
# We aim to have all conversions covered.
# =========================================================================== #
def test_various_conversions
  _ = BASE_DIR+'/'+File.basename(TEST_FILE)
  e 'Now converting that '+sfancy('.mp3')+' file into '+sfancy('.wav')+
    ' format:'
  mp3_to_wav _
  cliner
  e 'Now converting from '+sfancy('.mp3')+' into '+sfancy('.ogg')+
    ' format (mp3 to ogg):'
  mp3_to_ogg _
  cliner
  e 'Now converting from '+sfancy('.wav')+' into '+sfancy('.ogg')+
    ' format (wav to ogg):'
  wav_to_ogg Dir[BASE_DIR+'*.wav'].sample # Into .ogg format.
  cliner
  e 'Next we will test '+sfancy('MultimediaParadise.wav_to_mp3()')
  MultimediaParadise.wav_to_mp3(wav?)
  cliner
  e 'Now converting from '+sfancy('.wav')+' into '+sfancy('.mp3')+
    ' format (wav to mp3):'
  # We must make sure that WAV_FILE exists.
  unless File.exist? WAV_FILE
    extract_audio = ExtractAudio.new(MP4_FILE).path?
    convert(extract_audio, :to_wav)  
  end
  convert(WAV_FILE, :to_mp3)
  convert(WAV_FILE, :mp3)
  convert(MP3_FILE, :to_aac)
  convert(MP3_FILE, :to_wv)
  convert(MP3_FILE, :to_wma)
  convert(MP3_FILE, :to_opus) # Creating an .opus file here.
  e 'We created some audio files in `'+sdir(BASE_DIR)+'`.'
  e
  e 'Next testing MultimediaParadise.remove_metadata()'
  MultimediaParadise.remove_metadata MAIN_FILE
  e 'Next testing CutAudio functionality.'
  MultimediaParadise.cut_audio(MAIN_FILE, '30%')
end; test_various_conversions # Invoke it at once.